"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.authConfig = authConfig;
exports.addAction = addAction;
exports.addMiddleware = addMiddleware;
exports.authExpress = authExpress;
exports.authRoutes = authRoutes;
exports.authMiddleware = authMiddleware;

var _typeof2 = _interopRequireDefault(require("@babel/runtime/helpers/typeof"));

var _passport = _interopRequireDefault(require("passport"));

var _database = _interopRequireDefault(require("./database"));

var _loginWithApi = _interopRequireDefault(require("./middleware/login-with-api"));

var _defendWithJwt = _interopRequireDefault(require("./middleware/defend-with-jwt"));

var _route = require("./route");

var _config = require("./config");

/**
 * Set authentication module's config or get value from config
 */
function authConfig(configOrKey) {
  var type = (0, _typeof2.default)(configOrKey);

  if (type === 'string') {
    return (0, _config.getConfig)(configOrKey);
  } else if (type === 'object') {
    (0, _config.setConfig)(configOrKey);
  } else return (0, _config.getConfig)();
}

function addAction(type, version, action) {
  if (!action && typeof version === 'function') {
    action = version;
    version = 'default';
  }

  version = version.replace('.', '_');
  (0, _config.changeConfig)('functions.' + version + '.actions.' + type, action);
}

function addMiddleware(type, version, middleware) {
  if (!middleware && typeof version === 'function') {
    middleware = version;
    version = 'default';
  }

  version = version.replace('.', '_');
  (0, _config.changeConfig)('functions.' + version + '.middleware.' + type, middleware);
}
/**
 * Add passport to express middleware
 */


function authExpress(app) {
  _passport.default.serializeUser(function (user, done) {
    done(null, user);
  });

  _passport.default.deserializeUser(function (user, done) {
    done(null, user);
  });

  app.use(_passport.default.initialize());
  app.use(_passport.default.session());
}
/**
 * Create default routes for login if we are lazy
 */


function authRoutes(app, version) {
  var options = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : null;

  if ((0, _typeof2.default)(version) === 'object' && !options) {
    options = version;
    version = null;
  }

  version = version || '';
  options = options || {};
  (0, _route.loginRoute)(app, options.login || {}, version);
  (0, _route.logoutRoute)(app, options.logout || {}, version);
  (0, _route.refreshTokenRoute)(app, options.refreshToken || {}, version);
}

function authMiddleware(type, options, middleware) {
  switch (type) {
    case 'auth.api':
      return (0, _loginWithApi.default)(options, middleware);

    case 'defend.jwt':
      return (0, _defendWithJwt.default)(options, middleware);
  }
}