"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.appendInputValidation = appendInputValidation;
exports.byUsernameAndPassword = byUsernameAndPassword;
exports.successLoginInMiddleware = successLoginInMiddleware;

var _bcrypt = _interopRequireDefault(require("bcrypt"));

var _bluebird = _interopRequireDefault(require("bluebird"));

var _check = require("express-validator/check");

var _authentication = require("./../authentication");

var _database = _interopRequireDefault(require("./../database"));

var _logout = _interopRequireDefault(require("./../services/logout"));

var _device = _interopRequireDefault(require("./../services/device/device"));

var _token = _interopRequireDefault(require("./../helpers/token"));

var _user = require("./user");

function log(data) {
  var l = (0, _authentication.authConfig)('log');
  l(data);
}

;
var msg = "Invalid username or password";
var notActivated = "This user is not activated";

var validateToArray = function validateToArray(ret, data) {
  if (!data) return ret;

  if (Array.isArray(data)) {
    for (var i in data) {
      ret.push(data[i]);
    }
  } else ret.push(data);

  return ret;
};

function appendInputValidation(middleware, settings) {
  var inputs = (0, _authentication.authConfig)('validateInputs');
  middleware = validateToArray(middleware, function (req, res, next) {
    if (!req.body[settings.usernameField]) settings.invalidRequest(req, res, [settings.usernameField], settings);else next();
  });
  middleware = validateToArray(middleware, inputs.password((0, _check.check)(settings.passwordField)));
  middleware.push(function (req, res, next) {
    var errors = (0, _check.validationResult)(req);

    if (!errors.isEmpty()) {
      log('Invalid authenticate request', errors.array());
      settings.invalidRequest(req, res, errors.array(), settings);
    } else next();
  });
  return middleware;
}

function byUsernameAndPassword(username, password, done) {
  (0, _database.default)().repository('account').findByAccount(username).then(function (account) {
    if (account) {
      log('Account found ' + JSON.stringify(account));

      var getPassword = function getPassword() {
        return new _bluebird.default(function (resolve, reject) {
          (0, _database.default)().repository('password').findAllByIdentity(account.identity_id).then(function (existsPasswords) {
            var checkPassword = function checkPassword(key, pass) {
              if (!pass[key]) return resolve(null);

              _bcrypt.default.compare(password, pass[key].password, function (err, res) {
                if (res) {
                  log('Valid password found');
                  resolve(pass[key]);
                } else {
                  log('Invalid password');
                  checkPassword(key + 1, existsPasswords);
                }
              });
            };

            checkPassword(0, existsPasswords);
          }).catch(function (err) {
            reject(err);
          });
        });
      };

      var getIdentity = function getIdentity() {
        return new _bluebird.default(function (resolve, reject) {
          (0, _database.default)().repository('identity').findOneById(account.identity_id).then(function (identity) {
            if (!identity) reject('Identity not found');else resolve(identity);
          }).catch(function (err) {
            reject(err);
          });
        });
      };

      _bluebird.default.all([getPassword(), getIdentity()]).then(function (values) {
        var password = values[0],
            identity = values[1];
        if (!password) return done(null, false, {
          message: msg,
          code: 1
        });else if (!identity.activated) {
          console.log(notActivated);
          return done(null, false, {
            message: notActivated,
            code: 2
          });
        } else (0, _user.createObject)({
          identity: (0, _database.default)().getId(identity),
          password: (0, _database.default)().getId(password),
          account: (0, _database.default)().getId(account)
        }).then(function (user) {
          done(null, user);
        }).catch(function (err) {
          console.error(err);
          return done(null, false, {
            message: 'Something bad',
            code: -1
          });
        });
      }).catch(function (err) {
        console.error(err);
        done(null, false, {
          message: 'Something bad',
          code: -1
        });
      });
    } else {
      log("Account " + username + " not found");
      done(null, false, {
        message: msg,
        code: 3
      });
    }
  }).catch(function (err) {
    throw err;
  });
}

function successLoginInMiddleware(account, req, next, settings) {
  var deviceService = new _device.default();
  deviceService.user(account).create(req.body.device || {}).then(function (device) {
    account.device = device;
    account.id = (0, _database.default)().getId(account.user);
    req.logIn(account, {
      "session": settings && settings.session ? settings.session : false
    }, function (err) {
      if (err) {
        return next(err);
      } //Duplikálva a defender/jwt.jsbe


      req.token = function (type) {
        type = type || "jwt";
        var generator = (0, _token.default)(type);

        if (generator) {
          generator.user(account);
          return generator;
        } else {
          process.exit(1);
        }
      };

      req.logout = function () {
        var lo = new _logout.default();
        lo.user(account);
        lo.logout();
      };

      next();
    });
  });
}