'use strict';

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = _default;

var _passport = _interopRequireDefault(require("passport"));

var _passportJwt = require("passport-jwt");

var _lodash = _interopRequireDefault(require("lodash"));

var _logout = _interopRequireDefault(require("./../services/logout"));

var _jwt = _interopRequireDefault(require("./../services/token/jwt"));

var _database = _interopRequireDefault(require("./../database"));

var _token = _interopRequireDefault(require("./../helpers/token"));

var _config = require("./../config");

var _user = require("./../helpers/user");

var _version = require("./../helpers/version");

var _settings = require("./../helpers/settings");

var _strategy = require("passport-jwt/lib/strategy");

var defaultSettings = {
  'version': 'default',
  'check_expire': true
};
var versionSettings = {};

function log(data) {
  var l = (0, _config.getConfig)('log');
  l(data);
}

var strategyAdded;

function defineStrategy() {
  if (strategyAdded) return;
  var opts = {
    jwtFromRequest: _passportJwt.ExtractJwt.fromAuthHeaderAsBearerToken(),
    secretOrKey: process.env.JWT_SECRET || 'DqOlYSMMLZCr8JxSffaQ7Kpe7sd9s28M4UjJVyrvHD0GcUSqke1bCYoAXk7EHr1BKxy16xiUi0WEfKsFonxxHJeAwlva0qdN9L4b',
    passReqToCallback: true
  };
  strategyAdded = true;

  _passport.default.use(new _passportJwt.Strategy(opts, function (req, jwt_payload, done) {
    var settings,
        jwtService = new _jwt.default();

    if (req.tokenSettings) {
      settings = req.tokenSettings;
    } else settings = defaultSettings;

    var userData = jwtService.decryptData(jwt_payload.token_hash);
    (0, _database.default)().repository('identity').findOneById(userData.identity || 0).then(function (identity) {
      if (!identity) {
        log('User (' + userData.identity + ') from token not found');
        return done(null, false);
      } else if (!identity.activated) {
        log('This user is not activated in email');
        return done(null, false);
      }

      var seconds = new Date().getTime() / 1000,
          isExpired = settings.check_expire !== false && jwt_payload.expire > 0 && jwt_payload.expire < parseInt(seconds);

      if (isExpired) {
        log('This token is expired');
        return done(null, false);
      }

      (0, _user.createObject)(userData).then(function (user) {
        if (!user.device.disabled) {
          return done(null, user);
        } else {
          log('This device is logged out');
          done(null, false);
        }
      }).catch(function (err) {
        log(err);
        done(null, false);
      });
    }).catch(function (err) {
      log('ERROR: ' + err);
      done(null, false);
    });
  }));
}

;

function checkCanPass(middleware) {
  return function (req, res, next) {
    var opt = req.tokenSettings;

    if (opt.canGuest === true && !req.headers['authorization']) {
      next();
    } else {
      middleware(req, res, next);
    }
  };
}

var hasToken = function hasToken(req, res, next) {
  var middleware = (0, _version.middleware)('token', req);
  if (middleware) return middleware(req, res, next);else next();
};

var addUserFunctions = function addUserFunctions(req, res, next) {
  req.token = function (type) {
    type = type || "jwt";
    var generator = (0, _token.default)(type);

    if (generator) {
      generator.user(req.user);
      return generator;
    } else {
      process.exit(1);
    }
  };

  req.logout = function () {
    var lo = new _logout.default();
    lo.user(req.user);
    lo.logout();
  };

  next();
};

function _default(options, middleware) {
  var version = options && options.version || 'default';
  middleware = middleware || [];
  defineStrategy();
  middleware.push(function (req, res, next) {
    req.tokenSettings = _lodash.default.assign({}, defaultSettings, options);
    next();
  }, checkCanPass(_passport.default.authenticate('jwt', {
    session: false
  })), checkCanPass(hasToken), checkCanPass(addUserFunctions));
  return middleware;
}