"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = _default;

var _lodash = _interopRequireDefault(require("lodash"));

var _passport = _interopRequireDefault(require("passport"));

var _passportLocal = require("passport-local");

var _login = require("./../helpers/login");

var _config = require("./../config");

var _settings = require("./../helpers/settings");

var defaultSettings = {
  'version': 'default',
  'usernameField': 'username',
  'passwordField': 'password',
  'session': false,
  'invalidAuthenticate': function invalidAuthenticate(res) {
    return res.status(401).json({
      'success': false,
      'error': {
        'code': '4010101',
        'type': '',
        'title': 'Invalid username or password',
        'detail': 'Invalid username or password'
      }
    });
  },
  'invalidRequest': function invalidRequest(req, res, errors) {
    return res.status(422).json({
      'success': false,
      'error': {
        'code': '4220102',
        'type': '',
        'title': 'Invalid authorization request',
        'detail': 'Invalid authorization request: ' + JSON.stringify(errors)
      }
    });
  },
  'userIsNotActivated': function userIsNotActivated(res) {
    return res.status(401).json({
      'success': false,
      'error': {
        'code': '4010103',
        'type': '',
        'title': 'This user is not activated',
        'detail': 'This user is not activated'
      }
    });
  }
};
var versionSettings = {};

function log(data) {
  var l = (0, _config.getConfig)('log');
  l(data);
}

var strategyAdded = {};

function defineStrategy(settings) {
  if (strategyAdded[settings.version]) return;
  strategyAdded[settings.version] = true;

  _passport.default.use('local-' + settings.version, new _passportLocal.Strategy({
    usernameField: settings.usernameField,
    passwordField: settings.passwordField
  }, function (username, password, done) {
    (0, _login.byUsernameAndPassword)(username, password, done);
  }));
}

function _default(options, middleware) {
  var version = options && options.version || 'default';
  middleware = middleware || [];
  versionSettings = (0, _settings.defineSettings)(version, options, versionSettings, defaultSettings);
  var settings = (0, _settings.settingsByVersion)(version, versionSettings),
      ret = middleware || [];
  defineStrategy(settings);
  ret = (0, _login.appendInputValidation)(ret, settings);
  ret.push(function (req, res, next) {
    return _passport.default.authenticate('local-' + settings.version, function (err, account, msg) {
      if (!err && account) {
        log('Authentication success');
        (0, _login.successLoginInMiddleware)(account, req, next, settings);
      } else if (err) {
        log('Error: passport authenticate error');
        next(err);
      } else {
        log('Authentication failed');

        if (msg.code && msg.code == 2) {
          settings.userIsNotActivated(res);
        } else {
          settings.invalidAuthenticate(res);
        }
      }
    })(req, res, next);
  });
  return ret;
}