"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.loginRoute = loginRoute;
exports.refreshTokenRoute = refreshTokenRoute;
exports.logoutRoute = logoutRoute;

var _version = require("./helpers/version");

var _authentication = require("./authentication");

// Login
function loginRoute(app, options, version) {
  app.post((version ? '/' + version : '') + '/api/login', (0, _authentication.authMiddleware)('auth.api', {
    'version': version || null
  }, options.middleware), function (req, res, next) {
    (0, _version.action)('login', req, options.action)(req, res, next);
  });
} // Refresh token


var refreshToken = function refreshToken(req, res) {
  req.token().refresh(req.headers.authorization, req.body.refresh_token).then(function (token) {
    res.json({
      'success': true,
      'response': {
        'token': token
      }
    });
  }).catch(function (err) {
    console.error(err);
    return res.status(422).json({
      'success': false,
      'error': {
        "code": '4220103',
        "type": '',
        "title": 'Invalid refresh token',
        "detail": 'Invalid refresh token: ' + req.body.refresh_token
      }
    });
  });
};

function refreshTokenRoute(app, options, version) {
  app.post((version ? '/' + version : '') + '/api/refresh_token', (0, _authentication.authMiddleware)('defend.jwt', {
    'check_expire': false,
    'version': version || null
  }, options.middleware), options.action || refreshToken);
} //Logout


var logout = function logout(req, res) {
  req.logout();
  res.json({
    'error': false
  });
};

function logoutRoute(app, options, version) {
  app.get((version ? '/' + version : '') + '/api/logout', (0, _authentication.authMiddleware)('defend.jwt', {
    'version': version || null
  }, options.middleware), options.action || logout);
}