"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _classCallCheck2 = _interopRequireDefault(require("@babel/runtime/helpers/classCallCheck"));

var _createClass2 = _interopRequireDefault(require("@babel/runtime/helpers/createClass"));

var _bluebird = _interopRequireDefault(require("bluebird"));

var _jsonwebtoken = _interopRequireDefault(require("jsonwebtoken"));

var _crypto = _interopRequireDefault(require("crypto"));

var _config = require("./../../config");

var _database = _interopRequireDefault(require("./../../database"));

var _user2 = require("./../../helpers/user");

var _device = _interopRequireDefault(require("./../device/device"));

var _disable = _interopRequireDefault(require("./../device/disable"));

var tokenType = 'bearer';

var Token =
/*#__PURE__*/
function () {
  function Token() {
    (0, _classCallCheck2.default)(this, Token);
    this.deviceService = new _device.default();
    this.log = (0, _config.getConfig)('log');
  }

  (0, _createClass2.default)(Token, [{
    key: "user",
    value: function user(_user) {
      this.userObject = _user;
      this.deviceService.user(_user);
      return this;
    }
  }, {
    key: "generate",
    value: function generate(options) {
      var expire = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 0;
      var customContent = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : null;
      var self = this;
      var createDevice = new _bluebird.default(function (resolve) {
        self.deviceService.create(options).then(function (device) {
          resolve(device);
        });
      });
      return createDevice.then(function (device) {
        var expiredAt = expire || 0,
            jwtContent = customContent || {},
            object = (0, _user2.getAuthKeys)(self.userObject);
        object.device = (0, _database.default)().getId(device);
        object.timestamp = Date.now();
        jwtContent = {
          token_hash: self.encryptData(object)
        };
        if (expiredAt > 0) jwtContent["expire"] = Math.round(new Date().getTime() / 1000 + expiredAt);
        var refreshToken = generateRefreshToken(object.identity);
        return generateJsonWebToken(jwtContent, device, refreshToken).then(function (token) {
          var disable = new _disable.default();
          disable.device(device);
          return _bluebird.default.resolve({
            "token": token,
            "disable": disable
          });
        });
      });
    }
  }, {
    key: "encryptData",
    value: function encryptData(data) {
      return encrypt(JSON.stringify(data));
    }
  }, {
    key: "decryptData",
    value: function decryptData(data) {
      return JSON.parse(decrypt(data));
    }
  }, {
    key: "decode",
    value: function decode(token) {
      var decoded = _jsonwebtoken.default.decode(token);

      if (decoded) {
        return decoded;
      }

      return null;
    }
  }, {
    key: "refresh",
    value: function refresh(token, refreshToken) {
      var expire = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 0;
      var newRefreshToken = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : false;
      token = token.replace(tokenType, "").replace(" ", "");
      this.log('Try to refresh this token: ' + token + ' with ' + refreshToken);
      var decodedToken = this.decode(token);

      if (decodedToken) {
        if (decodedToken.iat) delete decodedToken.iat;
        var userData = JSON.parse(decrypt(decodedToken.token_hash));
        return (0, _database.default)().repository('device').findOneById(userData.device).then(function (device) {
          if (!device) return _bluebird.default.reject("Device not found for this token");else if (device.refresh_token != refreshToken) return _bluebird.default.reject("AAAA Invalid refresh token");
          var expiredAt = expire || 0;
          if (expiredAt > 0) decodedToken["expire"] = Math.round(new Date().getTime() / 1000 + expiredAt);
          var uploadRefreshToken = newRefreshToken ? generateRefreshToken(userData.identity) : refreshToken;
          return generateJsonWebToken(decodedToken, device, uploadRefreshToken).then(function (token) {
            if (!newRefreshToken) token.refresh_token = refreshToken;
            return token;
          });
        });
      } else return _bluebird.default.reject("Invalid token");
    }
  }]);
  return Token;
}();

exports.default = Token;

function generateJsonWebToken(jwtContent, device, refreshToken) {
  var log = (0, _config.getConfig)('log'),
      jwtKey = process.env.JWT_SECRET || 'DqOlYSMMLZCr8JxSffaQ7Kpe7sd9s28M4UjJVyrvHD0GcUSqke1bCYoAXk7EHr1BKxy16xiUi0WEfKsFonxxHJeAwlva0qdN9L4b',
      deviceId = (0, _database.default)().getId(device);
  return (0, _database.default)().repository('device').update(deviceId, {
    "refresh_token": refreshToken,
    "disabled": false
  }).then(function (device) {
    log("Token generated");
    return {
      "refresh_token": refreshToken,
      'token_type': tokenType.charAt(0).toUpperCase() + tokenType.slice(1),
      "access_token": _jsonwebtoken.default.sign(jwtContent, jwtKey),
      "expires_in": jwtContent["expire"] || 0
    };
  });
}

function generateRefreshToken(identity) {
  return _crypto.default.createHash('md5').update(identity + "_" + new Date().getTime()).digest("hex");
}

function encrypt(text) {
  var cipher = _crypto.default.createCipher('aes-256-cbc', 'd6F3Efeq');

  var crypted = cipher.update(text, 'utf8', 'hex');
  crypted += cipher.final('hex');
  return crypted;
}

function decrypt(text) {
  var decipher = _crypto.default.createDecipher('aes-256-cbc', 'd6F3Efeq');

  var dec = decipher.update(text, 'hex', 'utf8');
  dec += decipher.final('utf8');
  return dec;
}